@@ -1,38 +1,62 @@
-# -*- coding: utf-8 -*-
-"""
-    slixmpp.basexmpp
-    ~~~~~~~~~~~~~~~~~~
-
-    This module provides the common XMPP functionality
-    for both clients and components.
-
-    Part of Slixmpp: The Slick XMPP Library
-
-    :copyright: (c) 2011 Nathanael C. Fritz
-    :license: MIT, see LICENSE for more details
-"""
+# slixmpp.basexmpp
+# ~~~~~~~~~~~~~~~~~~
+# This module provides the common XMPP functionality
+# for both clients and components.
+# Part of Slixmpp: The Slick XMPP Library
+# :copyright: (c) 2011 Nathanael C. Fritz
+# :license: MIT, see LICENSE for more details
+from __future__ import annotations
 
 import asyncio
 import logging
 
+from typing import (
+    Dict,
+    Optional,
+    Union,
+    TYPE_CHECKING,
+)
+
 from slixmpp import plugins, roster, stanza
 from slixmpp.api import APIRegistry
 from slixmpp.exceptions import IqError, IqTimeout
 
-from slixmpp.stanza import Message, Presence, Iq, StreamError
+from slixmpp.stanza import (
+    Message,
+    Presence,
+    Iq,
+    StreamError,
+)
 from slixmpp.stanza.roster import Roster
 
 from slixmpp.xmlstream import XMLStream, JID
 from slixmpp.xmlstream import ET, register_stanza_plugin
 from slixmpp.xmlstream.matcher import MatchXPath
 from slixmpp.xmlstream.handler import Callback
-from slixmpp.xmlstream.stanzabase import XML_NS
+from slixmpp.xmlstream.stanzabase import (
+    ElementBase,
+    XML_NS,
+)
 
 from slixmpp.plugins import PluginManager, load_plugin
 
 
 log = logging.getLogger(__name__)
 
+
+from slixmpp.types import (
+    PresenceTypes,
+    MessageTypes,
+    IqTypes,
+    JidStr,
+    OptJidStr,
+)
+
+if TYPE_CHECKING:
+    # Circular imports
+    from slixmpp.pluginsdict import PluginsDict
+
+
 class BaseXMPP(XMLStream):
 
     """
@@ -44,6 +68,10 @@ class BaseXMPP(XMLStream):
                        is used during initialization.
     """
 
+    # This is technically not correct, but much more useful to typecheck
+    # than the internal use of the PluginManager API
+    plugin: PluginsDict
+
     def __init__(self, jid='', default_ns='jabber:client', **kwargs):
         XMLStream.__init__(self, **kwargs)
 
@@ -112,7 +140,7 @@ class BaseXMPP(XMLStream):
         self.use_presence_ids = True
 
         #: XEP-0359 <origin-id/> tag that gets added to <message/> stanzas.
-        self.use_origin_id = True
+        self.use_origin_id = False
 
         #: The API registry is a way to process callbacks based on
         #: JID+node combinations. Each callback in the registry is
@@ -221,7 +249,7 @@ class BaseXMPP(XMLStream):
                     self.plugin[name].post_init()
                 self.plugin[name].post_inited = True
 
-    def register_plugin(self, plugin, pconfig=None, module=None):
+    def register_plugin(self, plugin: str, pconfig: Optional[Dict] = None, module=None):
         """Register and configure  a plugin for use in this stream.
 
         :param plugin: The name of the plugin class. Plugin names must
@@ -236,9 +264,9 @@ class BaseXMPP(XMLStream):
         if not pconfig:
             pconfig = self.plugin_config.get(plugin, {})
 
-        if not self.plugin.registered(plugin):
+        if not self.plugin.registered(plugin):  # type: ignore
             load_plugin(plugin, module)
-        self.plugin.enable(plugin, pconfig)
+        self.plugin.enable(plugin, pconfig)  # type: ignore
 
     def register_plugins(self):
         """Register and initialize all built-in plugins.
@@ -251,10 +279,10 @@ class BaseXMPP(XMLStream):
         if self.plugin_whitelist:
             plugin_list = self.plugin_whitelist
         else:
-            plugin_list = plugins.__all__
+            plugin_list = plugins.PLUGINS
 
         for plugin in plugin_list:
-            if plugin in plugins.__all__:
+            if plugin in plugins.PLUGINS:
                 self.register_plugin(plugin)
             else:
                 raise NameError("Plugin %s not in plugins.__all__." % plugin)
@@ -271,32 +299,34 @@ class BaseXMPP(XMLStream):
         """Return a plugin given its name, if it has been registered."""
         return self.plugin.get(key, default)
 
-    def Message(self, *args, **kwargs):
+    def Message(self, *args, **kwargs) -> stanza.Message:
         """Create a Message stanza associated with this stream."""
         msg = Message(self, *args, **kwargs)
         msg['lang'] = self.default_lang
         return msg
 
-    def Iq(self, *args, **kwargs):
+    def Iq(self, *args, **kwargs) -> stanza.Iq:
         """Create an Iq stanza associated with this stream."""
         return Iq(self, *args, **kwargs)
 
-    def Presence(self, *args, **kwargs):
+    def Presence(self, *args, **kwargs) -> stanza.Presence:
         """Create a Presence stanza associated with this stream."""
         pres = Presence(self, *args, **kwargs)
         pres['lang'] = self.default_lang
         return pres
 
-    def make_iq(self, id=0, ifrom=None, ito=None, itype=None, iquery=None):
-        """Create a new Iq stanza with a given Id and from JID.
+    def make_iq(self, id: str = "0", ifrom: OptJidStr = None,
+                ito: OptJidStr = None, itype: Optional[IqTypes] = None,
+                iquery: Optional[str] = None) -> stanza.Iq:
+        """Create a new :class:`~.Iq` stanza with a given Id and from JID.
 
         :param id: An ideally unique ID value for this stanza thread.
                    Defaults to 0.
-        :param ifrom: The from :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The from :class:`~.JID`
                       to use for this stanza.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param itype: The :class:`~slixmpp.stanza.iq.Iq`'s type,
+        :param itype: The :class:`~.Iq`'s type,
                       one of: ``'get'``, ``'set'``, ``'result'``,
                       or ``'error'``.
         :param iquery: Optional namespace for adding a query element.
@@ -309,15 +339,17 @@ class BaseXMPP(XMLStream):
         iq['query'] = iquery
         return iq
 
-    def make_iq_get(self, queryxmlns=None, ito=None, ifrom=None, iq=None):
-        """Create an :class:`~slixmpp.stanza.iq.Iq` stanza of type ``'get'``.
+    def make_iq_get(self, queryxmlns: Optional[str] =None,
+                    ito: OptJidStr = None, ifrom: OptJidStr = None,
+                    iq: Optional[stanza.Iq] = None) -> stanza.Iq:
+        """Create an :class:`~.Iq` stanza of type ``'get'``.
 
         Optionally, a query element may be added.
 
         :param queryxmlns: The namespace of the query to use.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param ifrom: The ``'from'`` :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The ``'from'`` :class:`~.JID`
                       to use for this stanza.
         :param iq: Optionally use an existing stanza instead
                    of generating a new one.
@@ -332,15 +364,17 @@ class BaseXMPP(XMLStream):
             iq['from'] = ifrom
         return iq
 
-    def make_iq_result(self, id=None, ito=None, ifrom=None, iq=None):
+    def make_iq_result(self, id: Optional[str] = None,
+                       ito: OptJidStr = None, ifrom: OptJidStr = None,
+                       iq: Optional[stanza.Iq] = None) -> stanza.Iq:
         """
-        Create an :class:`~slixmpp.stanza.iq.Iq` stanza of type
+        Create an :class:`~.Iq` stanza of type
         ``'result'`` with the given ID value.
 
         :param id: An ideally unique ID value. May use :meth:`new_id()`.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param ifrom: The ``'from'`` :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The ``'from'`` :class:`~.JID`
                       to use for this stanza.
         :param iq: Optionally use an existing stanza instead
                    of generating a new one.
@@ -357,21 +391,23 @@ class BaseXMPP(XMLStream):
             iq['from'] = ifrom
         return iq
 
-    def make_iq_set(self, sub=None, ito=None, ifrom=None, iq=None):
+    def make_iq_set(self, sub: Optional[Union[ElementBase, ET.Element]] = None,
+                    ito: OptJidStr = None, ifrom: OptJidStr = None,
+                    iq: Optional[stanza.Iq] = None) -> stanza.Iq:
         """
-        Create an :class:`~slixmpp.stanza.iq.Iq` stanza of type ``'set'``.
+        Create an :class:`~.Iq` stanza of type ``'set'``.
 
         Optionally, a substanza may be given to use as the
         stanza's payload.
 
         :param sub: Either an
-                    :class:`~slixmpp.xmlstream.stanzabase.ElementBase`
+                    :class:`~.ElementBase`
                     stanza object or an
                     :class:`~xml.etree.ElementTree.Element` XML object
-                    to use as the :class:`~slixmpp.stanza.iq.Iq`'s payload.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+                    to use as the :class:`~.Iq`'s payload.
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param ifrom: The ``'from'`` :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The ``'from'`` :class:`~.JID`
                       to use for this stanza.
         :param iq: Optionally use an existing stanza instead
                    of generating a new one.
@@ -379,7 +415,7 @@ class BaseXMPP(XMLStream):
         if not iq:
             iq = self.Iq()
         iq['type'] = 'set'
-        if sub != None:
+        if sub is not None:
             iq.append(sub)
         if ito:
             iq['to'] = ito
@@ -391,7 +427,7 @@ class BaseXMPP(XMLStream):
                       condition='feature-not-implemented',
                       text=None, ito=None, ifrom=None, iq=None):
         """
-        Create an :class:`~slixmpp.stanza.iq.Iq` stanza of type ``'error'``.
+        Create an :class:`~.Iq` stanza of type ``'error'``.
 
         :param id: An ideally unique ID value. May use :meth:`new_id()`.
         :param type: The type of the error, such as ``'cancel'`` or
@@ -399,9 +435,9 @@ class BaseXMPP(XMLStream):
         :param condition: The error condition. Defaults to
                           ``'feature-not-implemented'``.
         :param text: A message describing the cause of the error.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param ifrom: The ``'from'`` :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The ``'from'`` :class:`~jid.JID`
                       to use for this stanza.
         :param iq: Optionally use an existing stanza instead
                    of generating a new one.
@@ -418,17 +454,19 @@ class BaseXMPP(XMLStream):
             iq['from'] = ifrom
         return iq
 
-    def make_iq_query(self, iq=None, xmlns='', ito=None, ifrom=None):
+    def make_iq_query(self, iq: Optional[stanza.Iq] = None, xmlns: str = '',
+                      ito: OptJidStr = None,
+                      ifrom: OptJidStr = None) -> stanza.Iq:
         """
-        Create or modify an :class:`~slixmpp.stanza.iq.Iq` stanza
+        Create or modify an :class:`~.Iq` stanza
         to use the given query namespace.
 
         :param iq: Optionally use an existing stanza instead
                    of generating a new one.
         :param xmlns: The query's namespace.
-        :param ito: The destination :class:`~slixmpp.xmlstream.jid.JID`
+        :param ito: The destination :class:`~.JID`
                     for this stanza.
-        :param ifrom: The ``'from'`` :class:`~slixmpp.xmlstream.jid.JID`
+        :param ifrom: The ``'from'`` :class:`~.JID`
                       to use for this stanza.
         """
         if not iq:
@@ -440,7 +478,7 @@ class BaseXMPP(XMLStream):
             iq['from'] = ifrom
         return iq
 
-    def make_query_roster(self, iq=None):
+    def make_query_roster(self, iq: Optional[stanza.Iq] = None) -> ET.Element:
         """Create a roster query element.
 
         :param iq: Optionally use an existing stanza instead
@@ -450,11 +488,14 @@ class BaseXMPP(XMLStream):
             iq['query'] = 'jabber:iq:roster'
         return ET.Element("{jabber:iq:roster}query")
 
-    def make_message(self, mto, mbody=None, msubject=None, mtype=None,
-                     mhtml=None, mfrom=None, mnick=None):
+    def make_message(self, mto: JidStr, mbody: Optional[str] = None,
+                     msubject: Optional[str] = None,
+                     mtype: Optional[MessageTypes] = None,
+                     mhtml: Optional[str] = None, mfrom: OptJidStr = None,
+                     mnick: Optional[str] = None) -> stanza.Message:
         """
         Create and initialize a new
-        :class:`~slixmpp.stanza.message.Message` stanza.
+        :class:`~.Message` stanza.
 
         :param mto: The recipient of the message.
         :param mbody: The main contents of the message.
@@ -476,11 +517,16 @@ class BaseXMPP(XMLStream):
             message['html']['body'] = mhtml
         return message
 
-    def make_presence(self, pshow=None, pstatus=None, ppriority=None,
-                      pto=None, ptype=None, pfrom=None, pnick=None):
+    def make_presence(self, pshow: Optional[str] = None,
+                      pstatus: Optional[str] = None,
+                      ppriority: Optional[int] = None,
+                      pto: OptJidStr = None,
+                      ptype: Optional[PresenceTypes] = None,
+                      pfrom: OptJidStr = None,
+                      pnick: Optional[str] = None) -> stanza.Presence:
         """
         Create and initialize a new
-        :class:`~slixmpp.stanza.presence.Presence` stanza.
+        :class:`~.Presence` stanza.
 
         :param pshow: The presence's show value.
         :param pstatus: The presence's status message.
@@ -500,11 +546,14 @@ class BaseXMPP(XMLStream):
         presence['nick'] = pnick
         return presence
 
-    def send_message(self, mto, mbody, msubject=None, mtype=None,
-                     mhtml=None, mfrom=None, mnick=None):
+    def send_message(self, mto: JID, mbody: Optional[str] = None,
+                     msubject: Optional[str] = None,
+                     mtype: Optional[MessageTypes] = None,
+                     mhtml: Optional[str] = None, mfrom: OptJidStr = None,
+                     mnick: Optional[str] = None):
         """
         Create, initialize, and send a new
-        :class:`~slixmpp.stanza.message.Message` stanza.
+        :class:`~.Message` stanza.
 
         :param mto: The recipient of the message.
         :param mbody: The main contents of the message.
@@ -520,11 +569,16 @@ class BaseXMPP(XMLStream):
         self.make_message(mto, mbody, msubject, mtype,
                           mhtml, mfrom, mnick).send()
 
-    def send_presence(self, pshow=None, pstatus=None, ppriority=None,
-                      pto=None, pfrom=None, ptype=None, pnick=None):
+    def send_presence(self, pshow: Optional[str] = None,
+                      pstatus: Optional[str] = None,
+                      ppriority: Optional[int] = None,
+                      pto: OptJidStr = None,
+                      ptype: Optional[PresenceTypes] = None,
+                      pfrom: OptJidStr = None,
+                      pnick: Optional[str] = None):
         """
         Create, initialize, and send a new
-        :class:`~slixmpp.stanza.presence.Presence` stanza.
+        :class:`~.Presence` stanza.
 
         :param pshow: The presence's show value.
         :param pstatus: The presence's status message.
@@ -537,11 +591,12 @@ class BaseXMPP(XMLStream):
         self.make_presence(pshow, pstatus, ppriority, pto,
                            ptype, pfrom, pnick).send()
 
-    def send_presence_subscription(self, pto, pfrom=None,
-                                   ptype='subscribe', pnick=None):
+    def send_presence_subscription(self, pto: JidStr, pfrom: OptJidStr = None,
+                                   ptype: PresenceTypes='subscribe', pnick:
+                                   Optional[str] = None):
         """
         Create, initialize, and send a new
-        :class:`~slixmpp.stanza.presence.Presence` stanza of
+        :class:`~.Presence` stanza of
         type ``'subscribe'``.
 
         :param pto: The recipient of a directed presence.
@@ -555,62 +610,62 @@ class BaseXMPP(XMLStream):
                            pnick=pnick).send()
 
     @property
-    def jid(self):
+    def jid(self) -> str:
         """Attribute accessor for bare jid"""
         log.warning("jid property deprecated. Use boundjid.bare")
         return self.boundjid.bare
 
     @jid.setter
-    def jid(self, value):
+    def jid(self, value: str):
         log.warning("jid property deprecated. Use boundjid.bare")
         self.boundjid.bare = value
 
     @property
-    def fulljid(self):
+    def fulljid(self) -> str:
         """Attribute accessor for full jid"""
         log.warning("fulljid property deprecated. Use boundjid.full")
         return self.boundjid.full
 
     @fulljid.setter
-    def fulljid(self, value):
+    def fulljid(self, value: str):
         log.warning("fulljid property deprecated. Use boundjid.full")
         self.boundjid.full = value
 
     @property
-    def resource(self):
+    def resource(self) -> str:
         """Attribute accessor for jid resource"""
         log.warning("resource property deprecated. Use boundjid.resource")
         return self.boundjid.resource
 
     @resource.setter
-    def resource(self, value):
+    def resource(self, value: str):
         log.warning("fulljid property deprecated. Use boundjid.resource")
         self.boundjid.resource = value
 
     @property
-    def username(self):
+    def username(self) -> str:
         """Attribute accessor for jid usernode"""
         log.warning("username property deprecated. Use boundjid.user")
         return self.boundjid.user
 
     @username.setter
-    def username(self, value):
+    def username(self, value: str):
         log.warning("username property deprecated. Use boundjid.user")
         self.boundjid.user = value
 
     @property
-    def server(self):
+    def server(self) -> str:
         """Attribute accessor for jid host"""
         log.warning("server property deprecated. Use boundjid.host")
         return self.boundjid.server
 
     @server.setter
-    def server(self, value):
+    def server(self, value: str):
         log.warning("server property deprecated. Use boundjid.host")
         self.boundjid.server = value
 
     @property
-    def auto_authorize(self):
+    def auto_authorize(self) -> Optional[bool]:
         """Auto accept or deny subscription requests.
 
         If ``True``, auto accept subscription requests.
@@ -620,11 +675,11 @@ class BaseXMPP(XMLStream):
         return self.roster.auto_authorize
 
     @auto_authorize.setter
-    def auto_authorize(self, value):
+    def auto_authorize(self, value: Optional[bool]):
         self.roster.auto_authorize = value
 
     @property
-    def auto_subscribe(self):
+    def auto_subscribe(self) -> bool:
         """Auto send requests for mutual subscriptions.
 
         If ``True``, auto send mutual subscription requests.
@@ -632,21 +687,21 @@ class BaseXMPP(XMLStream):
         return self.roster.auto_subscribe
 
     @auto_subscribe.setter
-    def auto_subscribe(self, value):
+    def auto_subscribe(self, value: bool):
         self.roster.auto_subscribe = value
 
-    def set_jid(self, jid):
+    def set_jid(self, jid: JidStr):
         """Rip a JID apart and claim it as our own."""
         log.debug("setting jid to %s", jid)
         self.boundjid = JID(jid)
 
-    def getjidresource(self, fulljid):
+    def getjidresource(self, fulljid: str):
         if '/' in fulljid:
             return fulljid.split('/', 1)[-1]
         else:
             return ''
 
-    def getjidbare(self, fulljid):
+    def getjidbare(self, fulljid: str):
         return fulljid.split('/', 1)[0]
 
     def _handle_session_start(self, event):
