@@ -1,11 +1,8 @@
-"""
-    Slixmpp: The Slick XMPP Library
-    Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
-    This file is part of Slixmpp.
-
-    See the file LICENSE for copying permission.
-"""
 
+# Slixmpp: The Slick XMPP Library
+# Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
+# This file is part of Slixmpp.
+# See the file LICENSE for copying permission.
 import unittest
 from queue import Queue
 from xml.parsers.expat import ExpatError
@@ -13,16 +10,16 @@ from xml.parsers.expat import ExpatError
 from slixmpp.test import TestTransport
 from slixmpp import ClientXMPP, ComponentXMPP
 from slixmpp.stanza import Message, Iq, Presence
+from slixmpp.stanza.error import Error
 from slixmpp.xmlstream import ET
 from slixmpp.xmlstream import ElementBase
 from slixmpp.xmlstream.tostring import tostring, highlight
 from slixmpp.xmlstream.matcher import StanzaPath, MatcherId, MatchIDSender
 from slixmpp.xmlstream.matcher import MatchXMLMask, MatchXPath
+from slixmpp.xmlstream.stanzabase import register_stanza_plugin
 
 import asyncio
-cls = asyncio.get_event_loop().__class__
 
-cls.idle_call = lambda self, callback: callback()
 
 class SlixTest(unittest.TestCase):
 
@@ -228,6 +225,10 @@ class SlixTest(unittest.TestCase):
                     "Stanza:\n%s" % str(stanza))
         else:
             stanza_class = stanza.__class__
+            # Hack to preserve namespaces instead of having jabber:client
+            # everywhere.
+            old_ns = stanza_class.namespace
+            stanza_class.namespace =stanza.namespace
             if not isinstance(criteria, ElementBase):
                 xml = self.parse_xml(criteria)
             else:
@@ -235,8 +236,8 @@ class SlixTest(unittest.TestCase):
 
             # Ensure that top level namespaces are used, even if they
             # were not provided.
-            self.fix_namespaces(stanza.xml, 'jabber:client')
-            self.fix_namespaces(xml, 'jabber:client')
+            self.fix_namespaces(stanza.xml)
+            self.fix_namespaces(xml)
 
             stanza2 = stanza_class(xml=xml)
 
@@ -279,6 +280,7 @@ class SlixTest(unittest.TestCase):
                 debug += "Given stanza:\n%s\n" % highlight(tostring(stanza.xml))
                 debug += "Generated stanza:\n%s\n" % highlight(tostring(stanza2.xml))
                 result = self.compare(xml, stanza.xml, stanza2.xml)
+            stanza_class.namespace = old_ns
 
             self.assertTrue(result, debug)
 
@@ -322,6 +324,7 @@ class SlixTest(unittest.TestCase):
         if not plugin_config:
             plugin_config = {}
 
+        self.mode = mode
         if mode == 'client':
             self.xmpp = ClientXMPP(jid, password,
                                    sasl_mech=sasl_mech,
@@ -332,6 +335,7 @@ class SlixTest(unittest.TestCase):
                                       plugin_config=plugin_config)
         else:
             raise ValueError("Unknown XMPP connection mode.")
+        self.xmpp._always_send_everything = True
 
         self.xmpp.connection_made(TestTransport(self.xmpp))
         self.xmpp.session_bind_event.set()
@@ -432,7 +436,9 @@ class SlixTest(unittest.TestCase):
             timeout      -- Time to wait in seconds for data to be received by
                             a live connection.
         """
+        self.wait_()
         self.xmpp.data_received(data)
+        self.wait_()
 
     def recv_header(self, sto='',
                           sfrom='',
@@ -610,8 +616,8 @@ class SlixTest(unittest.TestCase):
             self.fail("No stanza was sent.")
 
         xml = self.parse_xml(sent)
-        self.fix_namespaces(xml, 'jabber:client')
-        sent = self.xmpp._build_stanza(xml, 'jabber:client')
+        self.fix_namespaces(xml)
+        sent = self.xmpp._build_stanza(xml)
         self.check(sent, data,
                    method=method,
                    defaults=defaults,
@@ -621,10 +627,20 @@ class SlixTest(unittest.TestCase):
         loop = asyncio.get_event_loop()
         future = asyncio.ensure_future(self.xmpp.run_filters(), loop=loop)
         queue = self.xmpp.waiting_queue
-        print(queue)
         loop.run_until_complete(queue.join())
         future.cancel()
 
+    def wait_(self):
+        async def yield_some():
+            for i in range(100):
+                await asyncio.sleep(0)
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(yield_some())
+
+    def run_coro(self, coro):
+        loop = asyncio.get_event_loop()
+        return loop.run_until_complete(coro)
+
     def stream_close(self):
         """
         Disconnect the dummy XMPP client.
@@ -641,7 +657,7 @@ class SlixTest(unittest.TestCase):
     # ------------------------------------------------------------------
     # XML Comparison and Cleanup
 
-    def fix_namespaces(self, xml, ns):
+    def fix_namespaces(self, xml, ns=None):
         """
         Assign a namespace to an element and any children that
         don't have a namespace.
@@ -650,6 +666,10 @@ class SlixTest(unittest.TestCase):
             xml -- The XML object to fix.
             ns  -- The namespace to add to the XML object.
         """
+        if ns is None:
+            ns = 'jabber:client'
+            if self.xmpp:
+                ns = self.xmpp.default_ns
         if xml.tag.startswith('{'):
             return
         xml.tag = '{%s}%s' % (ns, xml.tag)
@@ -723,3 +743,10 @@ class SlixTest(unittest.TestCase):
 
         # Everything matches
         return True
+
+    def tearDown(self):
+        self.stream_close()
+        if getattr(self, "mode", None) == "component":
+            Error.namespace = 'jabber:client'
+            for st in Message, Iq, Presence:
+                register_stanza_plugin(st, Error)
