@@ -7,9 +7,6 @@ class TestReply(SlixTest):
     def setUp(self):
         self.stream_start(plugins=["xep_0461"])
 
-    def tearDown(self):
-        self.stream_close()
-
     def testFallBackBody(self):
         async def on_reply(msg):
             start = msg["feature_fallback"]["fallback_body"]["start"]
@@ -28,7 +25,7 @@ class TestReply(SlixTest):
             <message id="other-id" from="from@from.com/res">
               <reply xmlns="urn:xmpp:reply:0" id="some-id" />
               <body>&gt; quoted\nsome-body</body>
-                <fallback xmlns='urn:xmpp:feature-fallback:0' for='urn:xmpp:reply:0'>
+                <fallback xmlns='urn:xmpp:fallback:0' for='urn:xmpp:reply:0'>
                    <body start="0" end="8" />
                 </fallback>
             </message>
